/*$Id: lang_spice.h,v 26.51 2007/10/15 06:38:16 al Exp $ -*- C++ -*-
 * Copyright (C) 2006 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef LANG_SPICE_H
#define LANG_SPICE_H
#include "u_lang.h"
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
class LANG_SPICE : public LANGUAGE {
public:
  std::string name()const {return "spice";}
public: // override virtual, used by callback
  std::string arg_front()const {return " ";}
  std::string arg_mid()const {return "=";}
  std::string arg_back()const {return "";}

  // lang_spice_in.cc
public: // override virtual, called by commands
  void parse_comment(CS&, DEV_COMMENT*);
  void parse_command(CS&, DEV_DOT*);
  void parse_paramset(CS&, MODEL_CARD*);
  void parse_module(CS&, MODEL_SUBCKT*);
  void parse_instance(CS&, COMPONENT*);
public: // "local?", called by own commands
  void parse_module_body(CS&, MODEL_SUBCKT*, CARD_LIST*, const std::string&);
private: // local
  void parse_type(CS&, CARD*);
  void parse_args(CS&, CARD*);
  void parse_label(CS&, CARD*);
  void parse_ports(CS&, CARD*, int minnodes, int start, int num_nodes, bool all_new);
private: // compatibility hacks
  void parse_element_using_obsolete_callback(CS&, COMPONENT*);
  void parse_logic_using_obsolete_callback(CS&, COMPONENT*);

  // lang_spice_out.cc
private: // override virtual, called by print_item
  void print_paramset(OMSTREAM&, const MODEL_CARD*);
  void print_module(OMSTREAM&, const MODEL_SUBCKT*);
  void print_instance(OMSTREAM&, const COMPONENT*);
  void print_comment(OMSTREAM&, const DEV_COMMENT*);
  void print_command(OMSTREAM&, const DEV_DOT*);
private: // local
  void print_args(OMSTREAM&, const MODEL_CARD*);
  void print_type(OMSTREAM&, const COMPONENT*);
  void print_args(OMSTREAM&, const COMPONENT*);
  void print_label(OMSTREAM&, const COMPONENT*);
  void print_ports(OMSTREAM&, const COMPONENT*);
};
/*--------------------------------------------------------------------------*/
extern LANG_SPICE lang_spice;
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
