/*$Id: u_lang.h,v 26.52 2007/11/02 21:15:21 al Exp $ -*- C++ -*-
 * Copyright (C) 2006 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef U_LANG_H
#define U_LANG_H
#include "io_.h"
/*--------------------------------------------------------------------------*/
class COMPONENT;
class MODEL_SUBCKT;
class MODEL_CARD;
class CARD;
class DEV_COMMENT;
class DEV_DOT;
class CARD_LIST;
/*--------------------------------------------------------------------------*/
class LANGUAGE {
protected:
  const CARD* find_proto(const std::string&, const CARD*);
  void new__instance(CS& cmd, MODEL_SUBCKT* owner);
public:
  virtual ~LANGUAGE() {}
  virtual std::string name()const = 0;
public: // used by callback
  virtual std::string arg_front()const = 0;
  virtual std::string arg_mid()const = 0;
  virtual std::string arg_back()const = 0;
public: // real public interface
  virtual void parse_root(CS&, CARD_LIST*) {}
  virtual void parse_item(CS&, CARD*);
public: // called by commands and parse_item
  virtual void parse_comment(CS&, DEV_COMMENT*) {}
  virtual void parse_command(CS&, DEV_DOT*) {}
  virtual void parse_paramset(CS&, MODEL_CARD*) = 0;
  virtual void parse_module(CS&, MODEL_SUBCKT*) = 0;
  virtual void parse_instance(CS&, COMPONENT*) = 0;
public: // real public interface
  virtual void print_item(OMSTREAM&, const CARD*);
private: // called by print_item
  virtual void print_paramset(OMSTREAM&, const MODEL_CARD*) = 0;
  virtual void print_module(OMSTREAM&, const MODEL_SUBCKT*) = 0;
  virtual void print_instance(OMSTREAM&, const COMPONENT*) = 0;
  virtual void print_comment(OMSTREAM&, const DEV_COMMENT*) = 0;
  virtual void print_command(OMSTREAM&, const DEV_DOT*) = 0;
};
OMSTREAM& operator<<(OMSTREAM& o, LANGUAGE* x);
bool get(CS&, const std::string& key, LANGUAGE** val);
/*--------------------------------------------------------------------------*/
// This is for backward compatibility only.
// It will be removed in the future.
// Do not use in new code.
template <class T>
void print_pair(OMSTREAM& o, LANGUAGE* lang, const std::string& name,
		T value, bool test=true)
{
  assert(lang);
  if (test) {
    std::string front = lang->arg_front() + name + lang->arg_mid();
    o << front << value << lang->arg_back();
  }else{
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
